package cz.zcu.kiv.cosi.parkoviste.brana;

import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;

import cz.zcu.kiv.cosi.parkoviste.IKonfigurace;
import cz.zcu.kiv.cosi.parkoviste.IPrijeloOdjelo;
import cz.zcu.kiv.cosi.parkoviste.IPrijezdyOdjezdy;
import cz.zcu.kiv.cosi.parkoviste.IZapVyp;
import cz.zcu.kiv.cosi.parkoviste.parkoviste.ParkovistePlneEvent;

public class Brana implements IZapVyp, Runnable, ApplicationListener, IPrijeloOdjelo {
	private IPrijezdyOdjezdy parkoviste;
	
	private int frekvence;
	private static final int MS_ZA_MINUTU = 60000;
	
	private int prijelo;
	private int odjelo;

	private boolean beziSimulace;
	private boolean parkovistePlne;
	
	/* souet 1, ppsti typu - auto, bus, tir */
	private static final double[] PPSTI_VOZIDEL = {0.5, 0.2, 0.3};

	private static final int AUTO = 0;
	private static final int BUS = 1;
	private static final int TIR = 2;
	
	public Brana(IPrijezdyOdjezdy parkoviste, IKonfigurace konfigurace) {
		this.parkoviste = parkoviste;
		this.frekvence = konfigurace.getFrekvence();
		this.prijelo = 0;
		this.odjelo = 0;
	}

	public int getPrijelo() {
		return prijelo;
	}

	public int getOdjelo() {
		return odjelo;
	}
	
	@Override
	public void onApplicationEvent(ApplicationEvent event) {
	    if (event instanceof ParkovistePlneEvent) {
	    	setParkovistePlne();
	     }	    
	}
	
	private synchronized boolean isParkovistePlne(){
		return parkovistePlne;
	}

	private synchronized void setParkovistePlne(){
		parkovistePlne = true;
	}

	private synchronized void setParkovisteVolne(){
		parkovistePlne = false;
	}
	
	@Override
	public void run() {

		while(beziSimulace){

			try {
				Thread.sleep(Math.round((exponential(frekvence) * MS_ZA_MINUTU)));
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

			if (beziSimulace) {				
				if(jePrijezd()){
					prijezd(typVozidla());
				}else{
					odjezd(typVozidla());
				}
			}
		}
	}

	private void prijezd(int typ){

		if(isParkovistePlne()){
			return;
		}

		try {
			switch(typ){
			case AUTO: println("Prijezd auto."); parkoviste.prijeloAuto(); break;
			case BUS: println("Prijezd bus."); parkoviste.prijelBus(); break;
			case TIR: println("Prijezd tir."); parkoviste.prijelTir(); break;
			}

			prijelo++;
		} catch (Exception e) {
			//ignorujeme
		}
	}

	private void odjezd(int typ){

		try {
			switch(typ){
			case AUTO: println("Odjezd auto."); parkoviste.odjeloAuto(); break;
			case BUS: println("Odjezd bus."); parkoviste.odjelBus(); break;
			case TIR: println("Odjezd tir."); parkoviste.odjelTir(); break;
			}

			odjelo++;
			setParkovisteVolne();

		} catch (Exception e) {
			//ignorujeme
		}
	}

	public static double exponential(double lambda) {
		return -Math.log(1 - Math.random()) / lambda;
	}

	private static boolean jePrijezd(){

		if(Math.random() < 0.5){
			return true;
		}else{
			return false;
		}
	}

	private static int typVozidla(){

		double interval = 0;
		double ppst = Math.random();

		int vybranyTyp = 0;

		for(int i = 0; i < PPSTI_VOZIDEL.length; i++){
			//nastaveni hranice intervalu pro losovani
			interval += PPSTI_VOZIDEL[i];

			//pokud je generovane cislo z daneho intervalu
			if (ppst <= interval) {
				vybranyTyp = i;
				break;
			}
		}

		return vybranyTyp;
	}	

	@Override
	public void vyp() {
		beziSimulace = false;
		System.out.println("Brana vypnuta...");
	}

	@Override
	public void zap() {
		beziSimulace = true;
		(new Thread(this)).start();
		System.out.println("Brana zapnuta...");
	}

	private void println(String output) {
		System.out.println("(Brana) " + output);
	}
}
